#Region
#AutoIt3Wrapper_UseX64=n
#EndRegion

; PROGRAM:  CUBEITMOD V6.05 From original Evanery's CubeIt and Tobias's CubeitMod5b5
; FUNCTION: BFB GCode (Kisslicer) post-processor for CubeX Compatibility

#include <GUIConstantsEx.au3>
#include <WindowsConstants.au3>
#include <StaticConstants.au3>
#include <EditConstants.au3>
#include <Array.au3>
#include <MsgBoxConstants.au3>
#include <FileConstants.au3>


Global $InputFile, $SuckM227[4], $SuckM228[4], $PrimeM227[4], $PrimeM228[4], $SuckPrimeSpeed[4], $UseM227, $UseM228, $nSuckM227[4], $nSuckM228[4], $nPrimeM227[4], $nPrimeM228[4], $nSuckPrimeSpeed[4]
Global $ActualRPM, $NewXYF , $InfillPerimeterFactor[4], $SolidInterfaceFactor[4], $nFlow[4] , $nGain[4] , $ActualXYF , $nDiameter[4], $nExtrusionWidth, $nInfillExtrusionWidth , $nLayerThickness , $SpeedsAr[5], $FirstLayerSpeed
Global $ShortSolid , $ShortInfill , $SolidSpeed , $LoopSpeed , $PermimeterSpeed , $InfillSpeed , $InterfaceSpeed , $SupportSpeed , $SparseSpeed, $OldInterfaceSpeed
Global $Extruders = 0 , $Extruding = 0 , $MaxRPMfactor = 0, $WarningMSG="" , $FlagMsg=0 , $Filling=0, $Soliding =0 , $Perimetering = 0 , $Looping=0
Global $Interfacing = 0 , $Supporting =0, $Sparsing =0 , $Crowning = 0 , $Pillaring = 0 , $Rafting = 0 , $Pathing = 0
Global $nCubeitModMax[4] , $MinSpeed , $MaxSpeed, $SmallPeriLoopSpeed , $PeriLoopMin , $MinAngle
Global $Line ,$hInput , $nStack , $InfillStyle, $Once = 0 , $BedRoughness=0 , $BedOffsetZ=0 ,$PrintingFirstLayer=False , $UsedExt[4] = [0,0,0,0]
Global $HShift, $VShift , $Exp, $HighLimit
Global $PivotInfillFactor , $PivotInfillLength , $PivotSolidFactor , $PivotLength , $PivotLengthSolid[4]

If $CmdLine[0] > 0 Then
	; If Input File is Provided in Command Line, then Process without GUI (as a Kisslicer Post-Processor)
	; Call as CUBEITMOD V6.05 {InputFileName}
	;                Where {InputFileName} is the name of the file to process (will be saved as *.BAK)
	$InputFile = $CmdLine[1]
Else
	; Process Intercatively with GUI
	$InputFile = FileOpenDialog("CubeItMod V6.05: Please Select an Input File", @WorkingDir, "BFB Files (*.BFB)|All Files (*.*)", 1)
EndIf

; Scan the KISSLICER Comments in the Input File to get P & S for M227 and M228 values + other variables
If GetPrimeSuck() = 0 Then FatalError("Can't Find Materials Info in Print File." & @CRLF & "(Kisslicer Comments must be turned ON!)")

; Here we go!

StringToNumber();Change alphanumericals variables into numerical variables

ProcessFile() ;Main Job

Local $Extruder_Msg, $M227_Msg, $M228_Msg, $Variables_Msg ;Done Message

For $i=1 To $Extruders
   If $UsedExt[$i] = 1 And $nCubeitModMax[$i] > $MinSpeed Then
	$Extruder_Msg &= @CRLF & "Extruder " & $i & @CRLF & "M227 P" & $PrimeM227[$i] &  " S" & $SuckM227[$i] & @CRLF  & "M228 P" & $PrimeM228[$i] & " S" & $SuckM228[$i] & @CRLF & "Flow Tweak = " & $nFlow[$i] & @CRLF & "Solid% = " & String($SolidInterfaceFactor[$i]*100) & @CRLF & "Infill% = " & String($InfillPerimeterFactor[$i]*100) & @CRLF & "Speed Limit set to " & $nCubeitModMax[$i] / 60 & "mm/s" & @CRLF & "PivotLength for Solid path = " & round ($PivotLengthSolid[$i],2) & " mm" & @CRLF
	EndIf
 Next

 ;Check if very low RPM ( = X/Y speed incrased by more than 25%) was present on Kisslicer's BFB and if so display warning message.
 If $MaxRPMFactor > 1.25 Then
	$WarningMsg = "*** Warning *** : In order to fit the lowest Cubex's RMP" & @CRLF & "some X/Y speeds are incresed by " & String( 100*(Round(($MaxRPMFactor)-1,1))) & "%" & @CRLF & "You can try to add a pillar or incrase the layer thickness or incrase the speeds or reduce the min time layer to reduce this effect" & @CRLF
	$FlagMsg=48
 EndIf

; display M227/228 use
If $UseM227 = "1" then
   $M227_Msg = "M227 Turned ON"
Else
   $M227_Msg = "M227 Turned OFF"
EndIf

If $UseM228 = "1" then
   $M228_Msg = "M228 Turned ON"
Else
   $M228_Msg = "M228 Turned OFF"
EndIf
$Variables_Msg = @CRLF & "ShortSolid% = " & $ShortSolid & @CRLF & "ShortInfill% = " & $ShortInfill & @CRLF & "ShortLoopSpeed = " & $SmallPeriLoopSpeed & "mm/s" & @CRLF & "LoopSegLength = " & $PeriLoopMin & "mm" & @CRLF & "LoopSegAngle = " & $MinAngle & ""
; Done!
MsgBox($FlagMsg,"CubeItMod V6.05 Completed!", _
	$M227_Msg & @CRLF & $M228_Msg & @CRLF & _
	$Variables_Msg & @CRLF & _
	$WarningMsg & @CRLF & _
	$Extruder_Msg & @CRLF & _
	"New File:       " & $InputFile & @CRLF & _
	"Original File:  " & $InputFile & ".bak")

Exit

;--------------------------------------------------------------------------------------------------

Func GetPrimeSuck();Get Infos from original BFB file

Local $Extruder

	$Extruder_Count_Key = "; num_extruders = "
	$Extruder_Section_Key = "; *** Material Settings for Extruder "
	$M227S_Key = "; destring_suck = "
	$M227P_Key = "; destring_prime = "
	$M228S_Key = "; cost_per_cm3 = "
	$M228P_Key = "; sec_per_c_per_c = "
	$SuckPrimeSpeed_Key = "; destring_speed_mm_per_s = "
	$UseM227_key = "; use_destring = "
	$UseM228_key = "; fan_pwm = "
	$Bed_Key = "; bed_C = "
    $Flow_Key = "; flowrate_tweak = "
	$Gain1_Key = "; ext_gain_1 = "
	$Gain2_Key = "; ext_gain_2 = "
	$Gain3_Key = "; ext_gain_3 = "
    $ExtrusionWidth_Key = "; extrusion_width_mm = "
	$InfillExtrusionWidth_Key = "; infill_extrusion_width = "
	$LayerThickness_Key = "; layer_thickness_mm = "
	$Diameter_Key = "; fiber_dia_mm = "
	$Stack_Key = "; stacked_layers = "
    $Infill_Style_Key = "; infill_st_oct_rnd = "
	$BedOffsetZ_Key = "; bed_offset_z_mm = "
	$BedRoughness_Key = "; bed_roughness_mm = "
	$CubeitModMax_Key = "; destring_suck_speed_mm_per_s = "
    $ShortSolid_Key = "; ShortSolid% = "
	$ShortInfill_Key = "; ShortInfill% = "
	$SmallPeriLoopSpeed_Key = "; SmallLoopSpeed = "
	$PeriLoopMin_Key = "; LoopSegLength = "
	$MinAngle_Key = "; LoopSegAngle = "
	$PerimeterSpeed_Key = "; Perimeter Speed = "
	$LoopsSpeed_Key = "; Loops Speed = "
	$SolidSpeed_Key = "; Solid Speed = "
	$SparseSpeed_Key = "; Sparse Speed = "
	$FirstLayerSpeed_Key = "; first_layer_speed_mm_per_s = "
    $InterfaceSpeed_Key = "; InterfaceSpeed [mm/s] = "
	$InfillSpeed_Key = "; InfillSpeed [mm/s] = "
	$SupportSpeed_Key = "; SupportSpeed [mm/s] = "
    $Exp_Key = "; Exp = "
	$VShift_Key = "; VShift = "
	$HShift_Key = "; HShift = "
	$HighLimit_Key =  "; HighLimit = "


	$hInput = FileOpen($InputFile)

	$Hits = 0
	$Extruder = 0
	; Process the Input File's Header Line by Line
	While 1
		$line = FileReadLine($hInput)
		If @error = -1 Then ExitLoop

		 ;Get the number of used extruders
		If StringLeft($line, StringLen($Extruder_Count_Key)) = $Extruder_Count_Key Then
			$Extruders = ASC(StringMid($line, StringLen($Extruder_Count_Key) + 1, 1)) - 48
			$Hits += 1

		;Get the current extruder Number
		ElseIf StringLeft($line, StringLen($Extruder_Section_Key)) = $Extruder_Section_Key Then
			$Extruder = ASC(StringMid($line, StringLen($Extruder_Section_Key) + 1, 1)) - 48
			$Hits += 1

		 ;Get Gain S value for extruder 1
		ElseIf StringLeft($line, StringLen($Gain1_Key)) = $Gain1_Key Then
			$nGain[1] = number(StringTrimLeft($Line,StringLen($Gain1_Key)))
			$Hits += 1

		;Get Gain S value for extruder 2
		ElseIf StringLeft($line, StringLen($Gain2_Key)) = $Gain2_Key Then
			$nGain[2] = number(StringTrimLeft($Line,StringLen($Gain2_Key)))
			$Hits += 1

		;Get Gain S value for extruder 3
		ElseIf StringLeft($line, StringLen($Gain3_Key)) = $Gain3_Key Then
			$nGain[3] = number(StringTrimLeft($Line,StringLen($Gain3_Key)))
			$Hits += 1

		 ;Get Flow Tweak for current extruder
		ElseIf StringLeft($line, StringLen($Flow_Key)) = $Flow_Key Then
			$nFlow[$Extruder] = StringTrimLeft($Line,StringLen($Flow_Key))
			$Hits += 1

		 ;Get M227 S value for current extruder
		ElseIf StringLeft($line, StringLen($M227S_Key)) = $M227S_Key Then
			$SuckM227[$Extruder] = StringTrimLeft($Line,StringLen($M227S_Key))
			$Hits += 1

		;Get M227 P value for current extruder
		ElseIf StringLeft($line, StringLen($M227P_Key)) = $M227P_Key Then
			$PrimeM227[$Extruder] = StringTrimLeft($Line,StringLen($M227P_Key))
			$Hits += 1

		 ;Get M228 S value for current extruder
		ElseIf StringLeft($line, StringLen($M228S_Key)) = $M228S_Key Then
			$SuckM228[$Extruder] = StringTrimLeft($Line,StringLen($M228S_Key))
			$Hits += 1

		 ;Get M228 P value for current extruder
		ElseIf StringLeft($line, StringLen($M228P_Key)) = $M228P_Key Then
			$PrimeM228[$Extruder] = StringTrimLeft($Line,StringLen($M228P_Key))
			$Hits += 1

		 ;Get the speed of Suck/Prime for current extruder, used to change RPM while "Filling"
		ElseIf StringLeft($line, StringLen($SuckPrimeSpeed_Key)) = $SuckPrimeSpeed_Key Then
			$SuckPrimeSpeed[$Extruder] = StringTrimLeft($Line,StringLen($SuckPrimeSpeed_Key))
			$InfillPerimeterFactor[$Extruder]= (number($SuckPrimeSpeed[$Extruder]) / 100)
			if $InfillPerimeterFactor[$Extruder] <= 0.01 then FatalError ( "Infill% must be > 1, if you dont want to use it set 100, set more than 100 to increase the quantity of material deposed while ""Filling"" and less than 100 to decrease the quantity of material deposed while ""Filling""")
			$Hits += 1

		 ;Get the Bed temp to use to change RPM while extruding Solid
		ElseIf StringLeft($line, StringLen($Bed_Key)) = $Bed_Key Then
			$SolidInterfaceFactor[$Extruder] = (number(StringTrimLeft($Line,StringLen($Bed_Key)))/100)
			if $SolidInterfaceFactor[$Extruder] = 0 then FatalError ( "S/I/R% must be > 0, if you dont want to use it set 100, set more than 100 to increase the quantity of material deposed while ""making Solid"" and less than 100 to decrease quantity of material deposed while ""making Solid""")
			$Hits += 1

		 ;Get the Diameter Filament on each extruder
		ElseIf StringLeft($line, StringLen($Diameter_Key)) = $Diameter_Key Then
			$nDiameter[$Extruder] = number(StringTrimLeft($Line,StringLen($Diameter_Key)))
			$Hits += 1

		 ; Get Extrusion width
	    ElseIf StringLeft($line, StringLen($ExtrusionWidth_Key)) = $ExtrusionWidth_Key Then
		$nExtrusionWidth = number(StringTrimLeft($Line,StringLen($ExtrusionWidth_Key)))
			$Hits += 1

		 ; Get Infill Extrusion width
	    ElseIf StringLeft($line, StringLen($InfillExtrusionWidth_Key)) = $InfillExtrusionWidth_Key Then
		$nInfillExtrusionWidth = number(StringTrimLeft($Line,StringLen($InfillExtrusionWidth_Key)))
			$Hits += 1

		 ; Get Bed Roughness
	    ElseIf StringLeft($line, StringLen($BedRoughness_Key)) = $BedRoughness_Key Then
		$BedRoughness = number(StringTrimLeft($Line,StringLen($BedRoughness_Key)))
			$Hits += 1

		 ; Get Bed Offset Z
	    ElseIf StringLeft($line, StringLen($BedOffsetZ_Key)) = $BedOffsetZ_Key Then
		$BedOffsetZ = number(StringTrimLeft($Line,StringLen($BedOffsetZ_Key)))
			$Hits += 1

		 ; Get Layer Thickness
	    ElseIf StringLeft($line, StringLen($LayerThickness_Key)) = $LayerThickness_Key Then
		$nLayerThickness = number(StringTrimLeft($Line,StringLen($LayerThickness_Key)))
			$Hits += 1

		 ; Get ShortSolid% for RPM reduction while extruding paths shorters than PivotLength
		 ElseIf StringLeft($line, StringLen($ShortSolid_Key)) = $ShortSolid_Key Then
		$ShortSolid = number(StringTrimLeft($Line,StringLen($ShortSolid_Key)))
  		if $ShortSolid > 100 or $ShortSolid < -200 Then FatalError ("ShortSolid% must be > -200  and < 100")
		   $Hits += 1

	  	 ; Get ShortInfill% for RPM reduction while extruding paths shorters than PivotInfillLength
	    ElseIf StringLeft($line, StringLen($ShortInfill_Key)) = $ShortInfill_Key Then
		$ShortInfill = number(StringTrimLeft($Line,StringLen($ShortInfill_Key)))
		if $ShortInfill > 100 or $ShortInfill < 0 Then FatalError ("ShortInfill% must be > -200  and < 100")
		   $Hits += 1

		 ; Get PeriLoopSpeed for Speed Reduction on small Loops
	    ElseIf StringLeft($line, StringLen($SmallPeriLoopSpeed_Key)) = $SmallPeriLoopSpeed_Key Then
		$SmallPeriLoopSpeed = number(StringTrimLeft($Line,StringLen($SmallPeriLoopSpeed_Key)))
			$Hits += 1

		 ; Get PeriLoopMin for Speed Reduction on small Loops
	    ElseIf StringLeft($line, StringLen($PeriLoopMin_Key)) = $PeriLoopMin_Key Then
		$PeriLoopMin = number(StringTrimLeft($Line,StringLen($PeriLoopMin_Key)))
			$Hits += 1

		 ; Get MinAngle for Speed Reduction on small Loops
	    ElseIf StringLeft($line, StringLen($MinAngle_Key)) = $MinAngle_Key Then
		$MinAngle = number(StringTrimLeft($Line,StringLen($MinAngle_Key)))
			$Hits += 1

			; Get HShift for S_I_Factor and InfillFactor calculation
		 ElseIf StringLeft($line, StringLen($HShift_Key)) = $HShift_Key Then
		$HShift = number(StringTrimLeft($Line,StringLen($HShift_Key)))
  		if $HShift > 4 or $HShift < -2 Then FatalError ("HShift must be >= -2  and =< 4")
		   $Hits += 1

		   ; Get Exp for S_I_Factor and InfillFactor calculation
		 ElseIf StringLeft($line, StringLen($Exp_Key)) = $Exp_Key Then
		$Exp = number(StringTrimLeft($Line,StringLen($Exp_Key)))
  		if $Exp > 5 or $Exp < 1 Then FatalError ("Exp must be >= 1  and =< 5")
		   $Hits += 1

		   ; Get VShift for S_I_Factor and InfillFactor calculation
		 ElseIf StringLeft($line, StringLen($VShift_Key)) = $VShift_Key Then
		$VShift = number(StringTrimLeft($Line,StringLen($VShift_Key)))
  		if $VShift > 1.2 or $VShift < 1 Then FatalError ("VShift must be >= 1  and =< 1.2")
		   $Hits += 1

		   ; Get HighLimit for S_I_Factor and InfillFactor calculation
		 ElseIf StringLeft($line, StringLen($HighLimit_Key)) = $HighLimit_Key Then
		$HighLimit = number(StringTrimLeft($Line,StringLen($HighLimit_Key)))
  		if $HighLimit > 1.2 or $HighLimit < 1 Then FatalError ("HighLimit must be >= 1  and =< 1.2")
		   $Hits += 1

		 ; Get Perimeter Speed
	    ElseIf StringLeft($line, StringLen($PerimeterSpeed_Key)) = $PerimeterSpeed_Key Then
		$SpeedsAr[0] = number(StringTrimLeft($Line,StringLen($PerimeterSpeed_Key)))
		$PerimeterSpeed = $SpeedsAr[0]
			$Hits += 1

		 ; Get Loops Speed
	    ElseIf StringLeft($line, StringLen($LoopsSpeed_Key)) = $LoopsSpeed_Key Then
		$SpeedsAr[1] = number(StringTrimLeft($Line,StringLen($LoopsSpeed_Key)))
	    $LoopSpeed = $SpeedsAr[1]
		$Hits += 1

		 ; Get Solid Speed
	    ElseIf StringLeft($line, StringLen($SolidSpeed_Key)) = $SolidSpeed_Key Then
		$SpeedsAr[2] = number(StringTrimLeft($Line,StringLen($SolidSpeed_Key)))
		$SolidSpeed = $SpeedsAr[2]
			$Hits += 1

		 ; Get Sparse Speed
	    ElseIf StringLeft($line, StringLen($SparseSpeed_Key)) = $SparseSpeed_Key Then
		$SpeedsAr[3] = number(StringTrimLeft($Line,StringLen($SparseSpeed_Key)))
		$SparseSpeed = $SpeedsAr[3]
			$Hits += 1

		 ; Get First Layer Speed
	    ElseIf StringLeft($line, StringLen($FirstLayerSpeed_Key)) = $FirstLayerSpeed_Key Then
		$FirstLayerSpeed = number(StringTrimLeft($Line,StringLen($FirstLayerSpeed_Key)))
			$Hits += 1

		 ; Get Interface Speed
	    ElseIf StringLeft($line, StringLen($InterfaceSpeed_Key)) = $InterfaceSpeed_Key Then
		$SpeedsAr[4] = number(StringTrimLeft($Line,StringLen($InterfaceSpeed_Key)))
			$InterfaceSpeed = $SpeedsAr[4]
			if $InterfaceSpeed < 1 or $InterfaceSpeed > 60 Then
			FatalError ( "Interface Speed setting error" & @CRLF & "Look into Printer G-Gode/Prefix, the setting should look like this : " & @CRLF & "; InterfaceSpeed [mm/s] = 25" & @CRLF & " a value in the range 15-35 should be good for this parameter")
			EndIf
			$Hits += 1

		 ; Get Support Speed
	    ElseIf StringLeft($line, StringLen($SupportSpeed_Key)) = $SupportSpeed_Key Then
		$SupportSpeed = number(StringTrimLeft($Line,StringLen($SupportSpeed_Key)))
			$Hits += 1

		 ; Get how many layers are stacked
		 ElseIf StringLeft($line, StringLen($Stack_Key)) = $Stack_Key Then
		 $nStack = number(StringTrimLeft($Line,StringLen($Stack_Key)))
			$Hits += 1

		 ; Check infill style if Straight
		 ElseIf StringLeft($line, StringLen($Infill_Style_Key)) = $Infill_Style_Key Then
		 $InfillStyle = number(StringTrimLeft($Line,StringLen($Infill_Style_Key)))
			if $InfillStyle > 0 Then
			   If $Once = 0 then
				  MsgBox ( 48, "Infill Style not Straight" , "Cubeitmod has better control on Straight Infill Style." & @CRLF & @CRLF & "You're still free to choose another style." & @CRLF & @CRLF & "This message is automatically closed after 4 seconds.", 4 )
				  $Once +=1
			   Endif
			EndIf
			$Hits += 1

		 ;Get the destring/suck speed for current extruder, used to limit maximum Speed that CubeitMod can write
		ElseIf StringLeft($line, StringLen($CubeitModMax_Key)) = $CubeitModMax_Key Then
			$nCubeitModMax[$Extruder] = number(StringTrimLeft($Line,StringLen($CubeitModMax_Key))) * 60
			$Hits += 1

		; If = 0 destring M227 OFF, if =1 destring M227 ON
	    ElseIf StringLeft($line, StringLen($UseM227_Key)) = $UseM227_Key Then
		$UseM227 = StringTrimLeft($Line,StringLen($UseM227_Key))
			$Hits += 1

		 ; If = 0 destring M228 OFF, if =1 destring M228 ON, if >1 Error
	    ElseIf StringLeft($line, StringLen($UseM228_Key)) = $UseM228_Key Then
		$UseM228 = StringTrimLeft($Line,StringLen($UseM228_Key))
			$Hits += 1

		 ; Stop the search at the end ot the header
		 Elseif StringInStr ($Line, "Main G-Code")>0 Then
		 $MinSpeed =  _ArrayMin($SpeedsAr) * 60
		 $MaxSpeed = _ArrayMax($SpeedsAr)
		 	For $i=1 to $Extruders
			   if $MaxSpeed > (($nCubeitModMax[$i]/60) - 10) Then
			   FatalError("Speed selection Error." & @CRLF & "All the MaxSpeeds used into the Matl tabs must be 10 mm/s or more faster than the other speeds ( Perimeter, Loops, Solid, Support, Infill, Sparse Infill and Interface)")
			   EndIf
			Next

		 ExitLoop
		 EndIf

	WEnd

	FileClose($hInput)

	Return $Hits
EndFunc

Func ProcessFile();Main Job

Local $TxtExtruder, $OldRPM, $nExtr, $S_I_Factor , $OldReducedRPM
Local $FirstAfterWarming = 0 , $LastShort = 0 , $ReducedRPM , $X08act = 0
Local $YPos , $ZPos , $X0 , $X1 , $X2 , $Y0 , $Y1 , $Y2  , $FilePos , $SolidLength = 0 , $InterfaceSpeedPos
Local $OldLine , $InfillLength = 0 , $InfillFactor , $Layer , $PeriLoopLength = 0 , $PeriLoopAngle

	FileMove($InputFile, $InputFile & ".bak", 1)
	$hInput = FileOpen($InputFile & ".bak")
	$hOutput = FileOpen($InputFile, 2)


	; Process the Input File Line by Line
	While 1
		$line = FileReadLine($hInput)
		If @error = -1 Then ExitLoop

		; Eliminate Blank Lines
		If $line = "" Then ContinueLoop


		; checking Kisslicer version
		if StringLeft ($line,Stringlen("; version ")) = "; version " Then
			if StringLeft ($line,Stringlen("; version 1.5")) <> "; version 1.5" Then
			   MsgBox(0,"ERROR","This version of CubeitMod works only with Kisslicer 1.5, subscribe to Kisslicetalk forum and download the latest versions from the ""Official release Dropbox"" folder")
			Exit
			Endif
		 Endif

	    ;Look if is printing the first layer
			if StringLeft ($line,Stringlen("; BEGIN_LAYER_OBJECT z=")) = "; BEGIN_LAYER_OBJECT z=" Then
			$Layer = Number (StringTrimLeft ($line,Stringlen("; BEGIN_LAYER_OBJECT z=")))
			if $Layer == $nLayerThickness + $BedRoughness + $BedOffsetZ then
			   $PrintingFirstLayer = True
			   Else
			   $PrintingFirstLayer = False
			   EndIf
			Endif


		; Get Working Extruder = $TxTExtruder and Set $FirstAfterWarming to 1
		If StringLeft ($line,36) =Stringleft("; *** Selecting and Warming Extruder",36) Then
		   $TxtExtruder = StringMid ($line,38,1)
		   $nExtr = Number($TxtExtruder)
		   $FirstAfterWarming = 1
		   $UsedExt[$nExtr] = 1
		EndIf

		If StringLeft ($line,22) =Stringleft("; *** Warming Extruder",22) Then
		   $TxtExtruder = StringMid ($line,24,1)
		   $nExtr = Number($TxtExtruder)
		   $FirstAfterWarming = 1
		   $UsedExt[$nExtr] = 1
		EndIf

		If StringLeft ($line,27) =Stringleft("; *** Cooling Same Extruder",27) Then
		   $FirstAfterWarming = 1
		EndIf


		 ; Set $Filling or $Soliding or $Perimetering or $Looping or $Interfacing or $Supporting or $Sparsing or $Rafting var to 1

		 ; this is for Infill and Stacked Infill
		 If ((StringInStr ( $Line , "Infill Path'")) > 1 ) or ((StringInStr ( $Line , "Infill'")) > 1 ) Then
			$Filling = 1
			$X08act = 1
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			$OldLine = $Line
			$FilePos = FileGetPos ( $hInput )
			$Line = FileReadLine ( $hInput )
			   if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine

		 ;this is for Solid
		 ElseIf ((StringInStr ( $Line , "'Solid Path'")) > 1)  or ((StringInStr ( $Line , "'Solid'")) > 1) Then
			$Soliding = 1
			$X08act = 1
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			$OldLine = $Line
			$FilePos = FileGetPos ( $hInput )
			$Line = FileReadLine ( $hInput )
			   if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf ((StringInStr ( $Line , "'Perimeter Path'")) > 1) or ((StringInStr ( $Line , "'Perimeter'")) > 1) Then
			$Perimetering = 1
			$X08act = 1
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			$OldLine = $Line
			$FilePos = FileGetPos ( $hInput )
			$Line = FileReadLine ( $hInput )
			   if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf  ((StringInStr ( $Line , "'Loop Path'")) > 1) or ((StringInStr ( $Line , "'Loop'")) > 1) Then
			$Looping = 1
			$X08act = 1
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			$OldLine = $Line
			$FilePos = FileGetPos ( $hInput )
			$Line = FileReadLine ( $hInput )
			   if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf  ((StringInStr ( $Line , "'Support Interface Path'")) > 1) Then
			$Interfacing = 1
			$X08act = 1
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			$OldLine = $Line
			$FilePos = FileGetPos ( $hInput )
			$Line = FileReadLine ( $hInput )
			   if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf  ((StringInStr ( $Line , "'Support (may Stack) Path'")) > 1)  Then
		 $Supporting = 1
		 $X08act = 1
		 FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
		 $OldLine = $Line
		 $FilePos = FileGetPos ( $hInput )
		 $Line = FileReadLine ( $hInput )
			if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf  ((StringInStr ( $Line , "'Crown Path'")) > 1) Then
		 $Crowning = 1
		 $X08act = 1
		 FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
		 $OldLine = $Line
		 $FilePos = FileGetPos ( $hInput )
		 $Line = FileReadLine ( $hInput )
			if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine

		 ElseIf  ((StringInStr ( $Line , "'Raft Path'")) > 1) Then
		 $Rafting = 1
		 $X08act = 1
		 FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
		 $OldLine = $Line
		 $FilePos = FileGetPos ( $hInput )
		 $Line = FileReadLine ( $hInput )
			if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine

		 ElseIf  ((StringInStr ( $Line , "'Prime Pillar Path'")) > 1) Then
		 $Pillaring = 1
		 $X08act = 1
		 FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
		 $OldLine = $Line
		 $FilePos = FileGetPos ( $hInput )
		 $Line = FileReadLine ( $hInput )
			if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 ElseIf  ((StringInStr ( $Line , "Path'")) > 1) and ((StringInStr ($line , "'Destring/Wipe/Jump Path'")) = 0 ) Then
		 $Pathing = 1
		 $X08act = 1
		 FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
		 $OldLine = $Line
		 $FilePos = FileGetPos ( $hInput )
		 $Line = FileReadLine ( $hInput )
			if StringLeft ( $Line , 4 ) = "G1 X" Then
			   $YPos = StringInStr ( $Line , "Y")
			   $ZPos = StringInStr ( $Line , "Z")
			   $X1 = number ( StringMid ( $Line , 5 , $Ypos-2))
			   $Y1  = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			   Else
			   MsgBox (1, "ERROR " & $Line, "No G1 Line")
			   EndIf
			FileSetPos ( $hInput , $FilePos, $FILE_BEGIN )
			$Line = $OldLine
		 Endif
		; Eliminate Comments and unused M227
		If StringLeft($line, 1) = ";" Or StringLeft ($line,4)="M227" Then ContinueLoop

	    ; Place M227 & M228 with S and P values from Kisslicer
	    If StringLeft($line,4) = "M55" & $TxtExtruder and $UseM227 = "1" Then
		   FileWriteLine($hOutput,"M227 P"& String (Int($nPrimeM227[$nExtr] * 2962))& " S" & String (Int($nSuckM227[$nExtr] * 2962)))
			If $UseM228 = "1" Then
			FileWriteLine($hOutput,"M228 P"& String (Int($nPrimeM228[$nExtr] * 2962))& " S" & String (Int($nSuckM228[$nExtr] * 2962)))
			EndIf
		 EndIf

		;Place the first M103 to Suck before first layer and avoid initial blob
	  	; change RPM to real RPMs that the Cubex can do
		If StringLeft($line,4) = "M"&$TxtExtruder&"08" Then
	;$X08act = 1	$X08act = 1
	    $OldRPM = Number ( Stringmid ($line , 7))
			   If $FirstAfterWarming = 1 Then
				  If $OldRPM < 1 then
				  $ActualRPM = 1
				  	If ($ActualRPM / $OldRPM)> $MaxRPMFactor Then
					 $MaxRPMFactor = ($ActualRPM/$OldRPM)
					 EndIf
				  Else
				  $ActualRPM = Round($OldRPM , 0)
				  EndIf
			   $line = "M103"
			   $FirstAfterWarming = 0
			   Else
				  If $OldRPM < 1 then
				  $ActualRPM = 1
					 If ($ActualRPM / $OldRPM)> $MaxRPMFactor Then
					 $MaxRPMFactor = ($ActualRPM/$OldRPM)
					 EndIf
				  Else
				  $ActualRPM = Round ($OldRPM , 0)
				  EndIf
			   ;$line = ("M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			   ContinueLoop
			   EndIf
	  EndIf

		 ; Set $Extruding var to 1 when extrusion starts
   If StringLeft($Line,4) = "M"&$TxtExtruder&"01" then $Extruding=1

		;Calculate and write F speed according to real RPM cubex value for Perimeter and Loop
	    ; look if "Filling" and if so change RPM with the InfillPeirmeterFactor
		; look if "Soliding" and if so calculate the length of each line then calculate the $S_I_Factor for the line
		; Permimeter, Solid and Infill speeds are calculated from the settigns values
		; other speeds are calculated form the values calculated by Kisslicer ( at the moment )
	  If StringLeft($line,4)="G1 X" and $Extruding=1 Then
		If $Filling = 1 Then

			If $InfillStyle = 0 then
			$YPos = StringInStr ( $Line , "Y")
			$ZPos = StringInStr ( $Line , "Z")
			$X2 = Number ( StringMid ( $Line , 5 , $Ypos-2))
			$Y2 = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
			$InfillLength = Sqrt ( ( $X2 - $X1)^2 + ( $Y2 - $Y1 )^2)
			$InfillFactor = ((-1)/($InfillPerimeterFactor[$nExtr]*( -$HShift + $InfillLength )^$Exp))+ $VShift
			   If StringInStr ($InfillFactor , "#IND") >1 then
			   $InfillFactor = 0.2
			   EndIf
			   If $InfillFactor <  0.2 then
			   $InfillFactor =  0.2
			   EndIf
			   If $InfillFactor > $HighLimit Then
			   $InfillFactor = $HighLimit
			   Endif
			   If $PrintingFirstLayer = False Then
			   $NewXYF = Round ($ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nStack * $nInfillExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $InfillFactor ),1)
			   $PivotInfillFactor = $ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nStack * $nInfillExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $nCubeitModMax[$nExtr])
			   $PivotInfillLength = ((-1/($InfillPerimeterFactor[$nExtr]*($PivotInfillFactor-$VShift)))^(1/$Exp))+$HShift
			   Elseif $PrintingFirstLayer = True Then
			   $NewXYF = Round (($ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) )* ($nDiameter[$nExtr])^2 /(11.82 * $nStack * $nInfillExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $InfillFactor ),1)
			   $PivotInfillFactor = $ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) * ($nDiameter[$nExtr])^2 /(11.82 * $nStack * $nInfillExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $nCubeitModMax[$nExtr]* $FirstLayerSpeed / $SpeedsAr[2])
			   $PivotInfillLength = ((-1/($InfillPerimeterFactor[$nExtr]*($PivotInfillFactor-$VShift)))^(1/$Exp))+$HShift
			   EndIf
			   If $InfillLength > $PivotInfillLength then
				  If $LastShort = 1 Then
				  FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
				  EndIf
			   $X1 = $X2
			   $Y1 = $Y2
			   $line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
			   $LastShort = 0
			   Else
			   $ReducedRPM = ($ActualRPM-1)*(100-$ShortInfill)/100/$PivotInfillLength*($InfillLength-$PivotInfillLength)+$ActualRPM
				  if $ReducedRPM < 1 then
				  $ReducedRPM = 1
				  EndIf
				  If $PrintingFirstLayer = False Then
				  $NewXYF = $nCubeitModMax[$nExtr]
				  Elseif $PrintingFirstLayer = True Then
				  $NewXYF = Round($nCubeitModMax[$nExtr] * $FirstLayerSpeed / $SpeedsAr[2],1)
				  EndIf
			   $X1 = $X2
			   $Y1 = $Y2
			   $ReducedRPM = Round ($ReducedRPM , 0)
			   $line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
				  If $LastShort = 1 And $X08act = 1 Or $LastShort = 0 Or $ReducedRPM <> $OldReducedRPM Then
				  FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ReducedRPM))
				  EndIf
			   $LastShort = 1
			   $X08act = 0
			   $OldReducedRPM = $ReducedRPM
			   EndIf
			Else
			$ActualXYF = Number ( StringTrimLeft( $line ,(StringInStr ( $line , "F"))))
			   If $ActualRPM = 1 Then
			   $NewXYF= Round ($ActualXYF /($OldRPM*$InfillPerimeterFactor[$nExtr]),1)
			   Else
			   $NewXYF = Round ($ActualRPM*$ActualXYF /($OldRPM * $InfillPerimeterFactor[$nExtr]),1)
			   EndIf
			$line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
			$LastShort = 0
			Endif
		 Elseif $Soliding = 1 or $Interfacing =1 or $Rafting = 1 then
		 $YPos = StringInStr ( $Line , "Y")
		 $ZPos = StringInStr ( $Line , "Z")
		 $X2 = Number ( StringMid ( $Line , 5 , $Ypos-2))
		 $Y2 = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
		 $SolidLength = Sqrt ( ( $X2 - $X1)^2 + ( $Y2 - $Y1 )^2)
		 $S_I_Factor = ((-1)/($SolidInterfaceFactor[$nExtr]*( - $HShift + $SolidLength)^$Exp))+ $VShift
			If StringInStr ($S_I_Factor , "#IND") >1 then
			$S_I_Factor = 0.2
			EndIf
			If $S_I_Factor < 0.2 Then
			$S_I_Factor = 0.2
			EndIf
			If $S_I_Factor > $HighLimit then
			$S_I_Factor = $HighLimit
			EndIf
			If $PrintingFirstLayer = False Then
			$NewXYF = Round ($ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $S_I_Factor ),1)
			$PivotSolidFactor = $ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $nCubeitModMax[$nExtr])
			$PivotLength = ((-1/($SolidInterfaceFactor[$nExtr]*($PivotSolidFactor-$VShift)))^(1/$Exp))+$HShift
			$PivotLengthSolid[$nExtr] = $PivotLength
			Elseif $PrintingFirstLayer = True Then
			$NewXYF = Round (($ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) )* ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $S_I_Factor ),1)
			$PivotSolidFactor = $ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $nCubeitModMax[$nExtr]* $FirstLayerSpeed / $SpeedsAr[2])
			$PivotLength = ((-1/($SolidInterfaceFactor[$nExtr]*($PivotSolidFactor-$VShift)))^(1/$Exp))+$HShift
			EndIf
			If $SolidLength > $PivotLength then
			   If $LastShort = 1 Then
			   FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			   EndIf
			$X1 = $X2
			$Y1 = $Y2
			$line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
			$LastShort = 0
			Else
			$ReducedRPM = ($ActualRPM-1)*(100-$ShortSolid)/100/$PivotLength*($SolidLength-$PivotLength)+$ActualRPM
			   if $ReducedRPM < 1 then
			   $ReducedRPM = 1
			   EndIf
			   If $PrintingFirstLayer = False Then
			   $NewXYF = $nCubeitModMax[$nExtr]
			   Elseif $PrintingFirstLayer = True Then
			   $NewXYF = Round($nCubeitModMax[$nExtr] * $FirstLayerSpeed / $SpeedsAr[2],1)
			   EndIf
			$X1 = $X2
			$Y1 = $Y2
			$ReducedRPM = Round ($ReducedRPM , 0)
			$line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
			   If $LastShort = 1 And $X08act = 1 Or $LastShort = 0 Or $ReducedRPM <> $OldReducedRPM Then
			   FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ReducedRPM))
			   EndIf
			$LastShort = 1
			$X08act = 0
			$OldReducedRPM = $ReducedRPM
			EndIf
		 ElseIf $Perimetering = 1 Or $Looping = 1 Then
		 $YPos = StringInStr ( $Line , "Y")
		 $ZPos = StringInStr ( $Line , "Z")
		 $X2 = Number ( StringMid ( $Line , 5 , $Ypos-2))
		 $Y2 = Number ( StringMid ( $Line , $YPos+1 , $ZPos-2))
		 $PeriLoopLength = Sqrt ( ( $X2 - $X1)^2 + ( $Y2 - $Y1 )^2)
		 If $PeriLoopLength > $PeriLoopMin Then
			If $LastShort = 1 Then
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			EndIf
			If $PrintingFirstLayer = False Then
			$NewXYF = Round ($ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr]),1)
			Elseif $PrintingFirstLayer = True Then
			$NewXYF = Round (($ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) ) * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr]),1)
			EndIf
		 $LastShort = 0
		 Else
			If $X08act = 1 Then
			   If $PrintingFirstLayer = False Then
			   $ReducedRPM = Round (((11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $SmallPeriLoopSpeed * 60) / ( $nDiameter[$nExtr])^2 ),0)
				  If $ReducedRPM < 1 Then
				  $ReducedRPM = 1
				  EndIf
			   $NewXYF = Round ($ReducedRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] ),1)
			   Elseif $PrintingFirstLayer = True Then
				  If $SmallPeriLoopSpeed > $FirstLayerSpeed Then
				  $SmallPeriLoopSpeed = $FirstLayerSpeed
				  EndIf
			   $ReducedRPM = Round (((11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $SmallPeriLoopSpeed * 60 *($nLayerthickness + $BedRoughness)) / ($nLayerThickness * ($nDiameter[$nExtr]^2))),0)
				  If $ReducedRPM < 1 Then
				  $ReducedRPM = 1
				  EndIf
			   $NewXYF = Round (($ReducedRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) )* ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] ),1)
			   EndIf
			FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ReducedRPM))
			$LastShort = 1
			$X08act = 0
			Else
			$PeriLoopAngle = ACos(((( $X1 - $X0 ) * ( $X2 - $X1 )) +  (( $Y1 - $Y0 ) * ( $Y2 - $Y1 ))) / ( Sqrt ((( $X1 - $X0 )^2) + (( $Y1 - $Y0 )^2)) *Sqrt ((( $X2 - $X1 )^2) + (( $Y2 - $Y1 )^2)))) * (180/3.14159)
			If $PeriLoopAngle < $MinAngle Then
			   If $PrintingFirstLayer = False Then
			   $NewXYF = Round ($ActualRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr]),1)
			   Elseif $PrintingFirstLayer = True Then
			   $NewXYF = Round (($ActualRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) ) * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr]),1)
			   EndIf
			   If $LastShort = 1 Then
			   FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ActualRPM))
			   EndIf
			$LastShort = 0
			Else
			   If $PrintingFirstLayer = False Then
			   $ReducedRPM = Round (((11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $SmallPeriLoopSpeed * 60) / ( $nDiameter[$nExtr])^2 ),0)
				  If $ReducedRPM < 1 Then
				  $ReducedRPM = 1
				  EndIf
			   $NewXYF = Round ($ReducedRPM * ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] ),1)
				  If $LastShort = 0 Then
				  FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ReducedRPM))
				  EndIf
			   $LastShort = 1
			   Elseif $PrintingFirstLayer = True Then
				  If $SmallPeriLoopSpeed > $FirstLayerSpeed Then
				  $SmallPeriLoopSpeed = $FirstLayerSpeed
				  EndIf
			   $ReducedRPM = Round (((11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] * $SmallPeriLoopSpeed * 60 *($nLayerthickness + $BedRoughness)) / ($nLayerThickness * ($nDiameter[$nExtr]^2))),0)
				  If $ReducedRPM < 1 Then
				  $ReducedRPM = 1
				  EndIf
			   $NewXYF = Round (($ReducedRPM * $nLayerThickness/($nLayerthickness + $BedRoughness) )* ($nDiameter[$nExtr])^2 /(11.82 * $nExtrusionWidth * $nLayerThickness * $nFlow[$nExtr] * $nGain[$nExtr] ),1)
				  If $LastShort = 0 Then
				  FileWriteLine ($hOutput,"M" & $TxtExtruder & "08 S" & String ($ReducedRPM))
				  EndIf
			   $LastShort = 1
			   EndIf
			EndIf
		 EndIf
	  EndIf
	  $X0 = $X1
	  $Y0 = $Y1
	  $X1 = $X2
	  $Y1 = $Y2
	  $line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
	  Else
	  $ActualXYF = Number ( StringTrimLeft( $line ,(StringInStr ( $line , "F"))))
		 If $ActualRPM = 1 Then
		 $NewXYF= Round ($ActualXYF/($OldRPM),1)
		 Else
		 $NewXYF = Round ($ActualRPM*$ActualXYF/$OldRPM,1)
		 EndIf
	  $line = StringLeft($line , StringinStr ( $line , "F")) & String ($NewXYF)
	  $LastShort = 0
	  Endif
   EndIf

		 ;At the end of the printing path some vars are set to 0
		 If StringLeft($line,4) = "M103" Then
			$Extruding = 0
			$Filling = 0
			$Soliding = 0
			$Perimetering = 0
			$Looping = 0
			$Interfacing = 0
			$Pathing = 0
			$Supporting = 0
			$Pillaring = 0
			$Rafting = 0
			$Crowning = 0

		 EndIf

		; Write the Line
		FileWriteLine($hOutput, $line)

	WEnd

	FileClose($hInput)
	FileClose($hOutput)

EndFunc

;Conversion of alphanumerical variables into numerical variables
Func StringToNumber()
   Local $Count
   $Count =0
   For $Count = 1 to 3
	  $nSuckM227[$Count] = Number ($SuckM227[$Count])
	  $nPrimeM227[$Count] = Number ($PrimeM227[$Count])
	  $nSuckM228[$Count] = Number ($SuckM228[$Count])
	  $nPrimeM228[$Count] = Number ($PrimeM228[$Count])
	  $nSuckPrimeSpeed[$Count] = Number ($SuckPrimeSpeed[$Count])
	  $nCubeitModMax[$Count] = Number ($nCubeitModMax[$Count])
	Next

   EndFunc




Func FatalError($msg)
		MsgBox(0, "CUBEITMOD V6.05 Fatal Error!", $msg)
		Exit
EndFunc
